package me.lensfrex.manager.data;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

public class ManagerTool {
    private final ArrayList<Student> studentData;

    public ManagerTool(ArrayList<Student> studentData) {
        this.studentData = studentData;
    }

    /**
     * 添加一个学生
     *
     * @param id    学生id，long格式
     * @param name  学生姓名
     * @param score 学生成绩，map中key为科目名称，value为相应的成绩
     */
    public void addStudent(long id, String name, Map<String, Integer> score) {
        Student student = new Student(id, name, score);
        studentData.add(student);
    }

    /**
     * 按名字查找学生
     *
     * @param name 学生姓名
     * @return 学生的信息，若无该学生则返回null
     */
    public Student getStudent(String name) {
        for (Student student : studentData) {
            if (student.getName().equals(name)) {
                return student;
            }
        }

        return null;
    }

    /**
     * 按学号查找学生
     *
     * @param id 学生学号
     * @return 学生信息，若无该学生则返回null
     */
    public Student getStudent(long id) {
        for (Student student : studentData) {
            if (student.getId() == id) {
                return student;
            }
        }

        return null;
    }

    public static final int SORT_COMPARE_BY_NAME = 0;
    public static final int SORT_COMPARE_BY_ID = 1;
    public static final int SORT_COMPARE_BY_GENERAL_SCORE = 2;

    /**
     * 对学生列表进行排序
     *
     * @param compareMethod 排序方式，可根据名字字典顺序（升序，0）、学生学号（升序，1）以及总成绩（降序，2）进行排序。
     * @return 排序后的学生列表
     */
    public ArrayList<Student> getSortedStudentList(int compareMethod) {
        ArrayList<Student> sortedStudentList = new ArrayList<>(studentData);
        switch (compareMethod) {
            case SORT_COMPARE_BY_NAME:
                sortedStudentList.sort(Student.compareByName);
                break;
            case SORT_COMPARE_BY_ID:
                sortedStudentList.sort(Student.compareById);
                break;
            case SORT_COMPARE_BY_GENERAL_SCORE:
                sortedStudentList.sort(Student.compareByGeneralScore);
                break;
            default:
                break;
        }
        return sortedStudentList;
    }

    /**
     * 获取全班某课程的成绩信息
     *
     * @param subject 欲获取的科目信息
     * @return 包含成绩信息的HashMap，key为学生姓名，value为该科成绩数据
     */
    public HashMap<String, Integer> getSubjectScores(String subject) {
        HashMap<String, Integer> scores = new HashMap<>();
        for (Student student : studentData) {
            scores.put(student.getName(), student.getScore().get(subject));
        }
        return scores;
    }

    /**
     * 获取全班所有同学的总成绩信息
     *
     * @return 包含全班所有同学成绩的HashMap，key为姓名，value为总成绩数据
     */
    public HashMap<String, Integer> getStudentScores() {
        HashMap<String, Integer> result = new HashMap<>();

        int scoreSum = 0;
        for (Student student : studentData) {
            for (int score : student.getScore().values()) {
                scoreSum += score;
            }
            result.put(student.getName(), scoreSum);
            scoreSum = 0;
        }
        return result;
    }
}
